//*****************************************************************************
//*****************************************************************************
//  FILENAME:  `@LIB_NAME`_USBUART.c
//   Version: 1.4, Updated on 2009/5/7 at 21:16:21
//  `@PSOC_VERSION`
//
//  DESCRIPTION:  This file describes the processing logic for the USBUART interface.  When
//                the USBUART interface is instantiated, the buffer will be set
//                and the interface will be started.
//
//                Note:
//                Global interrupts MUST be enable prior to calling
//                USBUART_Instantiate.
//
//-----------------------------------------------------------------------------
//      Copyright (c) Cypress MicroSystems 2004. All Rights Reserved.
//*****************************************************************************
//*****************************************************************************
#include "m8c.h"
#include "PSoCAPI.h" 
#include "`@LIB_NAME`_USBUART.h"
#include "`@LIB_NAME`.h"
#include "stdlib.h"

//define types of entering command
#define	COMMAND_RD	0x01
#define	COMMAND_CRD	0x02
#define	COMMAND_WR	0x03

//define the types of error
#define	ERROR_NONE			0x00
#define	ERROR_COMMAND		0x01
#define	ERROR_COUNT			0x02
#define	ERROR_ADDRESS		0x03
#define	ERROR_DATA			0x04
#define	ERROR_RO_OVERFLOW	0x05
#define	ERROR_RW_OVERFLOW	0x06

//define the processing states
#define	STM_READ			0x01
#define	STM_VERIFY			0x02
#define	STM_EXECUTE			0x03

//define the size o internal buffer USB-UART
#define	SIZE_BUFFER			32
// define the information string
#define USBUART_COMMAND_ERROR	"Command Error!"
#define USBUART_ERROR_COUNT		"Illegal Byte Count!"
#define USBUART_ERROR_ADDR		"Illegal Register Address!"
#define USBUART_ERROR_DATA		"Illegal Data!"
#define USBUART_RO_OUT_RANGE	"Read-Only Register Out-of-Range!"
#define USBUART_RW_OUT_RANGE	"Read/Write Register Out-of-Range!"
#define USBUART_WRITE_BYTES		" bytes was written to the RAM RW area"

BYTE	`@LIB_NAME`_bRegAddr;			//the register sddress
BYTE	`@LIB_NAME`_bLengthCommand;		//the length of reading command
BYTE	`@LIB_NAME`_arbCommand[SIZE_BUFFER];//buffer for storage of reading command
BYTE	`@LIB_NAME`_bStage;				//the stage of processing
BYTE 	`@LIB_NAME`_bTypeCommand;		//type of command(0x01-RD,0x02-CRD,0x03-WR,0x00-None)
BYTE	`@LIB_NAME`_bCountByte;			//the count bytes for the reading from RAM
BYTE	`@LIB_NAME`_bInitUSB;			//flag of device initialization
BYTE	`@LIB_NAME`_bActivity;			//flag of USB bus activity
BYTE	`@LIB_NAME`_bCountCRD;			//flag of the CRD command executing

			
//-----------------------------------------------------------------------------
//  FUNCTION NAME: USBUART_Instantiate(const `@LIB_NAME`_USBUART_ParameterBlock * thisBLK)
//
//  DESCRIPTION:  
//    This function starts up the USBUART interface.  Global interrupts should be 
//    enabled prior to this call. 
//
//-----------------------------------------------------------------------------
//
//  ARGUMENTS: 
//    thisBLK  => Pointer to ParameterBlock for this instance.
//
//  RETURNS: None
//
//  SIDE EFFECTS: 
//
//  THEORY of OPERATION or PROCEDURE: 
//  Set  the flag of device initialization. Zero the length of reading command and set the stage of processing
//  to 0 (reading command). Initialize  the register address.
//-----------------------------------------------------------------------------
void `@LIB_NAME`_USBUART_Instantiate( const `@LIB_NAME`_USBUART_ParameterBlock * thisBLK )
{
	USBUART_SetPowerStatus(thisBLK->bPowerStatus); // Power Status can be STATUS_BUS_POWERED or STATUS_SELF_POWERED
	`@LIB_NAME`_bInitUSB=USBUART_Init(); 	//set  the flag of device initialization
	`@LIB_NAME`_bRegAddr = 0;			 	// Initialize  the register address
	`@LIB_NAME`_bLengthCommand = 0x00;	 	//Zero the length of reading command
	`@LIB_NAME`_bStage = STM_READ;			//set the stage of processing to 0 (reading command).
	`@LIB_NAME`_bCountCRD = 0x00;			//Initialize  the flag of the CRD command executing
}
//-----------------------------------------------------------------------------
//void `@LIB_NAME`_USBUART_DataTransfer(const `@LIB_NAME`_USBUART_ParameterBlock * thisBLK)
//
//  DESCRIPTION:  
//    Read, verify and execute command is entered by user on the hyperterminal.
//
//-----------------------------------------------------------------------------
//
//  ARGUMENTS: 
//    thisBLK     => Pointer to ParameterBlock for this instance.
//
//  RETURNS: None
//
//  SIDE EFFECTS: 
//
//  THEORY of OPERATION or PROCEDURE: 
//  The function executes the initialization of USB-UART slave device. 
//   If device is not initializing, function waits the enumeration of device. 
//  After the successful initialization the function reads all data from PC 
//  and compares this string with defined format of command.
//
//  Conditionally there are three working process stages of function: 
//  reading the command, processing the command accordingly to the defined format, 
//  execution the command.   
//
// On first stage user enters command. 
// On second stage function verifies the entering command to the corresponding 
// of defined formats. 
// On third stage the detected function executes the command: sends/reads the 
// given count of bytes to/from the PC
//-----------------------------------------------------------------------------
void `@LIB_NAME`_USBUART_DataTransfer(const `@LIB_NAME`_USBUART_ParameterBlock * thisBLK)
{
	BYTE 	i,j,k;					//loop counters
	BYTE	arbReceivedCommand[SIZE_BUFFER]; //buffer for receiver of reading command, buffer for storage two characters of byte
									//and use as flag of keystroke "ENTER"
	int 	iDataTemp;				//temp data for writting to the RAM
	BYTE 	bSizeCommand;			//number characters in name command
	BYTE	bByteError;				//flag of illegal byte 
	BYTE	bReadData;				//for storage the byte data from RAM
	BYTE 	bLen;					//the length of reading bytes
	BYTE	bTypeError;				//type of error (0x01-Error Command; 0x02-Error Count;
									//0x03-Error Address;0x04-Error Data; 0x05-Error Overflow;
									//0x00-None)
	int		iDataAddress;			//define address RW or RO area (register address+number of bytes) and data for writting to the RAM
	BYTE	bCountByte;				//number bytes for writting to the RAM RW area
	BYTE	bTypeWR;				//type WR is set address only(FALSE), type WR iswrite bytes(TRUE)
//********************************************* READ COMMAND STAGE****************** ********************************//	 		
	
	
	if(!`@LIB_NAME`_bInitUSB) 
	{
			USBUART_SetPowerStatus(thisBLK->bPowerStatus); // Power Status can be STATUS_BUS_POWERED or STATUS_SELF_POWERED
			`@LIB_NAME`_bInitUSB=USBUART_Init(); //set the flag   device initialization
	}		
	else		
	{		
		`@LIB_NAME`_bActivity=USBUART_bCheckUSBActivity();	//set the flag  activity USB bus
		if (!`@LIB_NAME`_bActivity)	//if USB bus is not active
		{
			`@LIB_NAME`_bInitUSB=0x00;	
			`@LIB_NAME`_bStage = STM_READ; //transition to stage 0 (read command)
			`@LIB_NAME`_bLengthCommand = 0x00;	 	//Zero the length of reading command
			return;	
		} 		
		else 
		{
			bTypeError = ERROR_NONE;
			if (`@LIB_NAME`_bStage==STM_READ)  	//if it was enter anyone character	
			{					
				bLen = USBUART_bGetRxCount();	//define the length of reading command
				if (bLen) 
				{							
					USBUART_ReadAll(arbReceivedCommand); 	//Read all data from RX

					if(arbReceivedCommand[0]==0xa) 			//Ignore the LF symbole
						return ; 
						
					for(i=0;i<bLen;i++)
					{
						`@LIB_NAME`_arbCommand[`@LIB_NAME`_bLengthCommand] = arbReceivedCommand[i];			
																//if the entering character is not ENTER and not full the internal buffer
						if (((arbReceivedCommand[i]!=0xd)&&(arbReceivedCommand[i]!=0xa))&&(`@LIB_NAME`_bLengthCommand < SIZE_BUFFER))
						{						
							`@LIB_NAME`_bLengthCommand++; 	//increment the length of entering command
						}
						else 
						{
							if(`@LIB_NAME`_USBUART_CheckReady())
								USBUART_PutCRLF();				//transfer to the next row
							`@LIB_NAME`_bStage = STM_VERIFY; 	//transition to stage 1 (verify command)
							break;
						}
					}
				}					
		}	//end stage 0 (read command)


//************ *******************************DETECTING COMMAND STAGE ******************************************//
			 if (`@LIB_NAME`_bStage == STM_VERIFY)
			{
				`@LIB_NAME`_bTypeCommand = `@LIB_NAME`_USBUART_CommandType(`@LIB_NAME`_arbCommand);	//detecting command
				
				if (!`@LIB_NAME`_bTypeCommand)	//if user set illegal command
					bTypeError=ERROR_COMMAND;	//define the type error and return to Read Command Stage
				else								
					`@LIB_NAME`_bStage = STM_EXECUTE;  //initial command execute stage 			

				if(`@LIB_NAME`_bTypeCommand & 0x01)	//define the length of name command
					bSizeCommand=3;
				else
					bSizeCommand=4;
							
				for (i=bSizeCommand;i<(bSizeCommand+2);i++)   //read byte (count or address)
						arbReceivedCommand[i-bSizeCommand]=`@LIB_NAME`_arbCommand[i];
						
				iDataAddress = `@LIB_NAME`_USBUART_Str16ToByte(arbReceivedCommand); //convert string to the byte	
				
//**************************************Define count of bytes for RD and CRD commands**************************************//
																	//if entering command is RD or CRD
				if ((`@LIB_NAME`_bTypeCommand==COMMAND_RD)||(`@LIB_NAME`_bTypeCommand==COMMAND_CRD))	
				{
					if((iDataAddress==0xFFFF) || ((`@LIB_NAME`_arbCommand[i]!=0xd)&&(`@LIB_NAME`_arbCommand[i]!=0xa)))   
						bTypeError=ERROR_COUNT;	//error count for RD and CRD command
					else 	
					{
						`@LIB_NAME`_bCountByte=(BYTE)iDataAddress; 	//set count variable for RD and CRD command
						if(`@LIB_NAME`_bCountByte == 0x00)
							bTypeError=ERROR_COUNT;	//error count for RD and CRD command
						
						iDataAddress = `@LIB_NAME`_bCountByte + `@LIB_NAME`_bRegAddr; //determine the propriety of count bytes 
																			//(it is situated in RO area)
						if(iDataAddress > thisBLK->bRAMLen)	//define the correct of register address
							bTypeError=ERROR_RO_OVERFLOW;
					}
				}
//*********************************************Define register address for WR command*****************************************//
				if (`@LIB_NAME`_bTypeCommand==COMMAND_WR)	//if entering command is WR
				{
					j = 0;
					bCountByte = 0;
					bTypeWR = 0;
					bByteError = 0;
					if((iDataAddress==0xFFFF) ) 
						bTypeError=ERROR_ADDRESS;	//error register address
					else 
					{
						if((`@LIB_NAME`_arbCommand[i]==0xd)||(`@LIB_NAME`_arbCommand[i]==0xa))	//Type of WR command set address offset only
						{
							`@LIB_NAME`_bStage = STM_READ;	
							`@LIB_NAME`_bLengthCommand = 0;
						}
						else if (`@LIB_NAME`_arbCommand[i]==' ')	//type of WR command write bytes data
						{
							for(i=6;i<`@LIB_NAME`_bLengthCommand+1;i++)  //check for correct all data of WR command
								if((`@LIB_NAME`_arbCommand[i]!=0xd)&&(`@LIB_NAME`_arbCommand[i]!=0xa)&&(`@LIB_NAME`_arbCommand[i]!=' '))
								{				
									if(j>1) //if user entered more than two characters for byte data 
									{
										bByteError=1;	//set the flag for incorrect byte data
										break;
									}	
									arbReceivedCommand[j]=`@LIB_NAME`_arbCommand[i];//accumulate the characters for byta data
									++j;
									bTypeWR = 1;
								}
								else
								{
									if(j<2)
									{
										bByteError=1;	//set the flag for incorrect byte data
										break;
									}	
									j=0;
									bCountByte++;
									iDataTemp=`@LIB_NAME`_USBUART_Str16ToByte(arbReceivedCommand);	//convert string to the byte
									if(iDataTemp==0xFFFF) 	//if user entered not correct data
									{
										bByteError=1;	//set the flag for incorrect byte data
										break;								
									}	
								}
						}
						else 
						{
							bByteError=1; 	//set the flag for incorrect byte data
							bTypeError=ERROR_ADDRESS;	//set the type error - Error Address
						}
						if (bByteError==0)	//if byte data is correct
							`@LIB_NAME`_bRegAddr = (BYTE)iDataAddress;	//set the register address
						else if(bTypeError==ERROR_NONE)
							bTypeError=ERROR_DATA;	//else set the type error - Error Data
						
						iDataAddress = bCountByte + `@LIB_NAME`_bRegAddr;	//define the correct of register address (it is situated in RW area)
						if ((iDataAddress > thisBLK->bRWLen)&&(bTypeWR))
							bTypeError=ERROR_RW_OVERFLOW;
						else if(bByteError==0)
							`@LIB_NAME`_USBUART_PrCountByte(bCountByte);	
							
					}
				}
			}	//end stage 1 (verify command)
				
				
//************************* **********************EXECUTING COMMAND STAGE ******************************************//	
			 if ((`@LIB_NAME`_bStage == STM_EXECUTE) && (bTypeError==ERROR_NONE)) 
			{
				if (`@LIB_NAME`_bTypeCommand==COMMAND_CRD)	//executing CRD command
				{		
					bLen = USBUART_bGetRxCount();	//define the length of entering characters
					arbReceivedCommand[0] = 0x00;		//Zero  the buffer for user entered symbol
					if (bLen) 						//if user entered any symbols
						USBUART_ReadAll(arbReceivedCommand);	//read all			
						
					if((arbReceivedCommand[0]==0xa)&&(`@LIB_NAME`_bCountCRD==0x00))	//Ignore the LF symbole
						return ; 
						
					if((arbReceivedCommand[0]!=0xd)&&(arbReceivedCommand[0]!=0xa))	//if the entering symbol is not ENTER 
					{					
						for (i=0;i<`@LIB_NAME`_bCountByte;i++)	
						{
							bReadData = thisBLK->pbRAMRegs[i+`@LIB_NAME`_bRegAddr];	//read bytes from RAM
							if(`@LIB_NAME`_USBUART_CheckReady())
								USBUART_PutSHexByte(bReadData);
							if(`@LIB_NAME`_USBUART_CheckReady())
								USBUART_PutChar(' ');
						}
						`@LIB_NAME`_bCountCRD=0x01;		//Set the flag of the CRD command executing
						if(`@LIB_NAME`_USBUART_CheckReady())
							USBUART_PutCRLF();			//transfer to the next row
					}
					else	//transition to stage 0 (read command)
					{
						`@LIB_NAME`_bCountCRD = 0x00;	//Zero the flag of the CRD command executing
						`@LIB_NAME`_bStage = STM_READ;	//set the stage of processing to 0 (reading command).
						`@LIB_NAME`_bLengthCommand = 0;	//Zero the length of reading command
					}
				}	
										
				if (`@LIB_NAME`_bTypeCommand==COMMAND_RD)	//if reading command is RD
				{	
						for (i=0;i<`@LIB_NAME`_bCountByte;i++)	
						{
							bReadData = thisBLK->pbRAMRegs[i+`@LIB_NAME`_bRegAddr];//read bytes from RAM
							if(`@LIB_NAME`_USBUART_CheckReady())
								USBUART_PutSHexByte(bReadData);
							if(`@LIB_NAME`_USBUART_CheckReady())
								USBUART_PutChar(' ');
						}
						`@LIB_NAME`_bStage = STM_READ;	//set the stage of processing to 0 (reading command).
						`@LIB_NAME`_bLengthCommand = 0;	//Zero the length of reading command
						if(`@LIB_NAME`_USBUART_CheckReady())
							USBUART_PutCRLF();	//transfer to the next row
				}	
									
				if (`@LIB_NAME`_bTypeCommand==COMMAND_WR)	//if reading command is WR
				{
					j = 0;
					k = 0;
					for(i=6;i<`@LIB_NAME`_bLengthCommand+1;i++)
						if((`@LIB_NAME`_arbCommand[i]!=0xd)&&(`@LIB_NAME`_arbCommand[i]!=0xa)&&(`@LIB_NAME`_arbCommand[i]!=' '))
						{			
							arbReceivedCommand[j]=`@LIB_NAME`_arbCommand[i];//read the byte for writting to the RAM
							++j;
						}
						else
						{
							j=0;
													//convert string to the byte and write byte to the RAM	
							thisBLK->pbRAMRegs[k+`@LIB_NAME`_bRegAddr] =(BYTE)`@LIB_NAME`_USBUART_Str16ToByte(arbReceivedCommand);
							k++;
						}
					`@LIB_NAME`_bStage = STM_READ;	//set the stage of processing to 0 (reading command).
					`@LIB_NAME`_bLengthCommand = 0;	//Zero the length of reading command
				}			
			}	//end stage 2 (execute command)
			
			if(bTypeError!=ERROR_NONE)
				`@LIB_NAME`_USBUART_PrMessage(bTypeError);	//print error message if error type is not None
		}
	}	
			
}

//***************************DEFINE COMMAND TYPE*******************************//
BYTE `@LIB_NAME`_USBUART_CommandType(BYTE *pbCommand)
{
	BYTE i;
	BYTE bCommType;
	
	bCommType = 0x00;
	if(pbCommand[2]==' ')
	{
		if((pbCommand[0]=='R') && (pbCommand[1]=='D'))
		{
			bCommType=COMMAND_RD; 		// RD Command		
		}
		if((pbCommand[0]=='W') && (pbCommand[1]=='R'))
		{
			bCommType=COMMAND_WR;			// WR Command		
		}
	}	
	else 
	{
		if((pbCommand[0]=='C') && (pbCommand[1]=='R') && (pbCommand[2]=='D'))
		{
			bCommType=COMMAND_CRD; 		// CRD Command		
		}
	}
	return bCommType;
}

//***************************CONVERT STRING TO BYTE*******************************//
int `@LIB_NAME`_USBUART_Str16ToByte(char *pbStrNum)
{
	int iResult;
	BYTE bTemp,i;
	iResult = 0;
	for(i=0;i<2;i++)
	{
		bTemp=pbStrNum[i];
		if((bTemp>=0x30)&&(bTemp<=0x39)) //if characters is number
			bTemp-=0x30;
		else
		{
			bTemp&=0xDF; //do uppercase from lowercase character
			if((bTemp>=0x41)&&(bTemp<=0x46)) //if characters is 'A','B','C','D','E' or 'F'
				bTemp-=0x37;
			else
			{
				iResult=0xFFFF; //it is error flag
				break;
			}
		}
		if(i==0)
			bTemp=bTemp<<4;	//form the result
		iResult = iResult | bTemp;
	}
	return iResult;
}


//***************************PRINT INFORM MESSAGE*******************************//
void `@LIB_NAME`_USBUART_PrMessage(BYTE bError)
{	
	if (bError==ERROR_COMMAND) //print inform messages depending on the type of error
		USBUART_CPutString(USBUART_COMMAND_ERROR);
	else if (bError==ERROR_COUNT)
		USBUART_CPutString(USBUART_ERROR_COUNT);
	else if (bError==ERROR_ADDRESS)
		USBUART_CPutString(USBUART_ERROR_ADDR);	
	else if (bError==ERROR_DATA)
		USBUART_CPutString(USBUART_ERROR_DATA);
	else if (bError==ERROR_RO_OVERFLOW)
		USBUART_CPutString(USBUART_RO_OUT_RANGE);
	else
		USBUART_CPutString(USBUART_RW_OUT_RANGE);
	if(`@LIB_NAME`_USBUART_CheckReady())
		USBUART_PutCRLF();	//transfer to the next row
	`@LIB_NAME`_bLengthCommand=0;//zero the length of reading command
	`@LIB_NAME`_bStage = STM_READ; //transition to stage 0 (read command)
}

//***************************PRINT THE NUMBER BYTES ****************************//
void `@LIB_NAME`_USBUART_PrCountByte(BYTE bCuuntByte)
{
	if(CMX_USBUART_CheckReady())
		USBUART_PutSHexByte(bCuuntByte);
	if(CMX_USBUART_CheckReady())
		USBUART_CPutString(USBUART_WRITE_BYTES);
	if(`@LIB_NAME`_USBUART_CheckReady())
		USBUART_PutCRLF();	//transfer to the next row
	
}

//***************************CHECK THE USBUART READY*******************************//
BYTE `@LIB_NAME`_USBUART_CheckReady(void)
{
	int i;
	BYTE bTxReady;
	for(i=0;i<0x200;i++)
	{
		bTxReady=USBUART_bTxIsReady();
		if (bTxReady)
			break;
	}
	return bTxReady;
}
