//*****************************************************************************
//*****************************************************************************
//  FILENAME:  CMX_SWINTCONTLED.c
//  Version: 1.1, Updated on 2008/6/23 at 14:9:22
//  
//
//  DESCRIPTION:  This Driver controls the SWINTCONTLED parameters and returns
//                the percent duty cycle for the selected channel.
//-----------------------------------------------------------------------------
//      Copyright (c) Cypress MicroSystems 2007 All Rights Reserved.
//*****************************************************************************
//*****************************************************************************

#include "m8c.h"
#include "CMX_SWINTCONTLED.h"
#include "cmx.h"

	// Channel type header file
#include "CMX_SWPWM_CHAN.h"

//-----------------------------------------------------------------------------
//  FUNCTION NAME: SWINTCONTLED_Instantiate(const CMX_SWINTCONTLED_ParameterBlock * thisBLK)
//
//  DESCRIPTION: SWINTCONTLED Initialization
//
//-----------------------------------------------------------------------------
//
//  ARGUMENTS: 
//    thisBLK  => Pointer to device parameter block
//
//  RETURNS: None
//
//  SIDE EFFECTS: 
//
//  THEORY of OPERATION or PROCEDURE:
//
//-----------------------------------------------------------------------------
void CMX_SWINTCONTLED_Instantiate(const CMX_SWINTCONTLED_ParameterBlock * thisBLK)
{
    
}
//-----------------------------------------------------------------------------
//  FUNCTION NAME: SWINTCONTLED_SetValue(const CMX_SWINTCONTLED_ParameterBlock * thisBLK,BYTE bState)
//
//  DESCRIPTION: SWINTCONTLED Sets the LED's intensity which  decays during DecayTime 
//
//-----------------------------------------------------------------------------
//
//  ARGUMENTS: 
//    thisBLK       => Pointer to parameter block
//
//  RETURNS: None
//
//  SIDE EFFECTS: 
//
//  THEORY of OPERATION or PROCEDURE:  CMX_SWPWM_CHAN is used to set the  LED intensity. 
// Software  PWM's  DutyCycle is controlled using 
//  driver output variable
//-----------------------------------------------------------------------------
void CMX_SWINTCONTLED_SetValue(const CMX_SWINTCONTLED_ParameterBlock * thisBLK, BYTE bSWINTCONTLEDDutyCycle)
{	
	if (bSWINTCONTLEDDutyCycle>100)		// check the overrange
		bSWINTCONTLEDDutyCycle = 100;
	if (thisBLK->Mode==1)		// if LED is sinking
	{
		bSWINTCONTLEDDutyCycle = 100-bSWINTCONTLEDDutyCycle;
	}
		
    CMX_SWPWM_SetPulseWidth(thisBLK->SWINTCONTLED_ChannelID, bSWINTCONTLEDDutyCycle);   // set the LED's intensity
}
