;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: CMX_SWPWMTIMER_CHAN.asm
;;   @Version@
;;  
;;  DESCRIPTION: Low level SWPWM channel for SWPWM driver 
;;   contains the function for SWPWM Interrupt 
;;  Service Routine and the functions for SWPWM driver. 
;;
;;  Theory of Operation:
;;  The low level SWPWM code  (Timer8 ISR) 
;;  provides forming PWM signal for each
;;  driver instance with appropriate parameter Duty Cycle
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress MicroSystems 2000-2003. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "m8c.inc"
include "memory.inc"
include "driverdecl.inc"
include "SWPWM_Timer.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------

export  CMX_SWPWM_ISR
export _CMX_SWPWM_ISR

export  InitSWPWM
export _InitSWPWM

export  CMX_SWPWM_SetPulseWidth
export _CMX_SWPWM_SetPulseWidth

export _SWPWM
export _SWPWMDutyCycle

AREA InterruptRAM (RAM,REL,CON)

;------------------------
; Variable Allocation
;------------------------

_SWPWM :
 SWPWM :          BLK 1            ; master software PWM

_SWPWMDutyCycle :                  ; DutyCycles for each LED instance
 SWPWMDutyCycle : BLK 4
 
AREA UserModules (ROM, REL)

;----------------------------------------------------------------------------
;  FUNCTION NAME: void InitSWPWMTimer( void )
;
;  DESCRIPTION:
;   Initialization and Starting shared channel (Timer8 UM) function 
;  Settling modes of output pins
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     none
;
;  RETURNS:  
;     none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;----------------------------------------------------------------------------- 

 InitSWPWM:
_InitSWPWM:

    RAM_PROLOGUE RAM_USE_CLASS_1
    RAM_PROLOGUE RAM_USE_CLASS_2

    mov   A,0x21
    mov   X,0x01
    lcall SetDriveMode
    mov   A,0x23
    mov   X,0x01
    lcall SetDriveMode
    mov   A,0x25
    mov   X,0x01
    lcall SetDriveMode
    mov   A,0x27
    mov   X,0x01
    lcall SetDriveMode

    lcall SWPWM_Timer_EnableInt            ; Turn on JUST the Terminal count IRQ
	
    lcall SWPWM_Timer_Start

    RAM_EPILOGUE RAM_USE_CLASS_2
    RAM_EPILOGUE RAM_USE_CLASS_1
    ret

;-----------------------------------------------------------------------------
;  FUNCTION NAME: void CMX_SWPWM_ISR( void )
;
;  DESCRIPTION:
;    This interrupt handler of Timer8 UM is called every 8 ms. It reads all 
;    SWPWM pins and stores (stack by shifting left) readed value of each SWPWM
;    input in corresponding PinHistory array.
;    The PinHistory array for each SWPWM instance consists of two subArray
;    PinHistory_MSB and PinHistory_LSB. 
;    The instance's number of SWPWM input indexes of this array. 
;    
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     none
;
;  RETURNS:  
;     none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;----------------------------------------------------------------------------- 
 CMX_SWPWM_ISR: 
_CMX_SWPWM_ISR: 
    push A                    ;store A and X registers
    push X                    

    inc[SWPWM]                ;roll up master PWM register once

; SW PWM Engine Code and Data Output for SWPWM_17 driver Instance

    mov   A,[SWPWM]                         ; store PWM register value
    cmp   A,[SWPWMDutyCycle+ID_SWPWM_17]    ; PWM < DutyCycle_SWPWM_17 ?
    mov   A,[Port_2_Data_SHADE]
    jnc   SWPWMOff_SWPWM_17                 ; NO - set pin level to LOW
    or    A,(0x01 << 0x1)                   ; turning LED ON
    jmp   SWPWMEnd_SWPWM_17                 ; end of operation
SWPWMOff_SWPWM_17:
    and   A,~(0x01 << 0x1)                  ; turning LED OFF
SWPWMEnd_SWPWM_17:
    mov   [Port_2_Data_SHADE],A	
    mov   reg[0x2 << 2],A                   ; settling port_pin levels 

; SW PWM Engine Code and Data Output for SWPWM_19 driver Instance

    mov   A,[SWPWM]                         ; store PWM register value
    cmp   A,[SWPWMDutyCycle+ID_SWPWM_19]    ; PWM < DutyCycle_SWPWM_19 ?
    mov   A,[Port_2_Data_SHADE]
    jnc   SWPWMOff_SWPWM_19                 ; NO - set pin level to LOW
    or    A,(0x01 << 0x3)                   ; turning LED ON
    jmp   SWPWMEnd_SWPWM_19                 ; end of operation
SWPWMOff_SWPWM_19:
    and   A,~(0x01 << 0x3)                  ; turning LED OFF
SWPWMEnd_SWPWM_19:
    mov   [Port_2_Data_SHADE],A	
    mov   reg[0x2 << 2],A                   ; settling port_pin levels 

; SW PWM Engine Code and Data Output for SWPWM_21 driver Instance

    mov   A,[SWPWM]                         ; store PWM register value
    cmp   A,[SWPWMDutyCycle+ID_SWPWM_21]    ; PWM < DutyCycle_SWPWM_21 ?
    mov   A,[Port_2_Data_SHADE]
    jnc   SWPWMOff_SWPWM_21                 ; NO - set pin level to LOW
    or    A,(0x01 << 0x5)                   ; turning LED ON
    jmp   SWPWMEnd_SWPWM_21                 ; end of operation
SWPWMOff_SWPWM_21:
    and   A,~(0x01 << 0x5)                  ; turning LED OFF
SWPWMEnd_SWPWM_21:
    mov   [Port_2_Data_SHADE],A	
    mov   reg[0x2 << 2],A                   ; settling port_pin levels 

; SW PWM Engine Code and Data Output for SWPWM_23 driver Instance

    mov   A,[SWPWM]                         ; store PWM register value
    cmp   A,[SWPWMDutyCycle+ID_SWPWM_23]    ; PWM < DutyCycle_SWPWM_23 ?
    mov   A,[Port_2_Data_SHADE]
    jnc   SWPWMOff_SWPWM_23                 ; NO - set pin level to LOW
    or    A,(0x01 << 0x7)                   ; turning LED ON
    jmp   SWPWMEnd_SWPWM_23                 ; end of operation
SWPWMOff_SWPWM_23:
    and   A,~(0x01 << 0x7)                  ; turning LED OFF
SWPWMEnd_SWPWM_23:
    mov   [Port_2_Data_SHADE],A	
    mov   reg[0x2 << 2],A                   ; settling port_pin levels 


    mov   A,[SWPWM]
    cmp   A,99              ; check if PWM period has elapsed
    jnz   SWPWM_Continue
    mov   [SWPWM],0xFF        ; reseting master PWM register
SWPWM_Continue:

    pop   X                   ;restore A and X registers
    pop   A
    ret
;-----------------------------------------------------------------------------
;  FUNCTION NAME: BYTE CMX_SWPWM_SetPinHistory(BYTE bChannelID, int iValue);
;
;  DESCRIPTION:
;    Sets the 16-pin PinHistory value of addressed SWPWM instance.
;    LMM supporting.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    [X-3] => index of SWPWM instance
;    [X-4][X-5] => new value for PinHistory; 
;
;  RETURNS:  
;     none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;    The function changes reg[IDX_PP] 
;
;-----------------------------------------------------------------------------
 CMX_SWPWM_SetPulseWidth: 
_CMX_SWPWM_SetPulseWidth: 

    RAM_PROLOGUE RAM_USE_CLASS_1
    RAM_PROLOGUE RAM_USE_CLASS_3

    RAM_SETPAGE_IDX >SWPWMDutyCycle

    swap  A,X                      ; modifying function parameters
    mov   [X+SWPWMDutyCycle], A    ; update DutyCycle Registers

    RAM_EPILOGUE RAM_USE_CLASS_3
    RAM_EPILOGUE RAM_USE_CLASS_1

    ret

; end of file  CMX_SWPWMTIMER_CHAN.asm
