;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: I2CFLASH.asm
;;  @Version@
;;  Generated by PSoC Designer ver 4.2  b1013 : 02 September, 2004
;;
;;  DESCRIPTION: I2CFLASH Low level driver
;;  
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress MicroSystems 2003. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************
include "EzI2Cs.inc"
include "m8c.inc"
include "memory.inc"
include "i2cflash.inc"
include "SystemConst.inc"
include "flashblock.inc"

EzI2Cs_FlashBlockSize:   equ  FlashBlockSize            ; Set Flash block size
EzI2Cs_RAMBufferSize:    equ  (FlashBlockSize+1)        ; Set RAM buffer size
EzI2Cs_FLASH_ENABLE:     equ  TotalBlocksUsed           ; If Set Flash interface enabled
EzI2Cs_LARGE_FLASH:      equ  ((FlashSize & 0x8000)>>15); Set if Flash larger than 16K
EzI2Cs_FirstBlock:       equ  FirstConstBlock           ; First Flash blocke used
EzI2Cs_BlocksUsed:       equ  TotalBlocksUsed           ; Flash blocks used

EzI2Cs_blockId_2bytes:   equ  (FLASH_WRITE_STRUCT_SIZE - 7) ; Size of blockID (0/1 -> 1/2 bytes size)
EzI2Cs_ReadCount_2bytes: equ  (FLASH_READ_STRUCT_SIZE - EzI2Cs_blockId_2bytes - 4) ; Size of ReadCount (0/1 -> 1/2 bytes size)

// The following is only allowd if Flash programming
// is enabled.
IF (EzI2Cs_FLASH_ENABLE)

export    EzI2Cs_Execute_CMD


ENDIF




AREA InterruptRAM (RAM, REL, CON)

;-----------------------------------------------
; Variable Allocation
;-----------------------------------------------



;-----------------------------------------------
;  EQUATES and TABLES
;-----------------------------------------------


STATE_WR_ROM_ADDR:  equ  0x08      ; Wait for Secondary address on write
STATE_RD_ROM:       equ  0x0A      ; Read ROM Data

STATE_WR_ROM:       equ  0x0C      ; Write ROM (Not supported at this time)

STATE_MASK:         equ  0x0E
STATE_MASK2:        equ  0x0F      ; State Mask

FLASH_MODE_FLAG:    equ  0x40      ; Set if in Flash mode.

I2C_FLASH_PAGE_OFFSET:   equ  0x00       // Zero except for 29xxx



;@PSoC_UserCode_INIT@ (Do not change this line.)
;---------------------------------------------------
; Insert your custom declarations below this banner
;---------------------------------------------------

;------------------------
;  Constant Definitions
;------------------------


;------------------------
; Variable Allocation
;------------------------


;---------------------------------------------------
; Insert your custom declarations above this banner
;---------------------------------------------------
;@PSoC_UserCode_END@ (Do not change this line.)




AREA UserModules (ROM, REL, CON)


IF (EzI2Cs_FLASH_ENABLE)

;-----------------------------------------------------------------------------
;  FUNCTION NAME: EzI2Cs_Execute_CMD
;
;  DESCRIPTION:
;     Execute flash command
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: none
;
;  RETURNS:   0 => Invalid command or completed incorrectly
;             1 => Valid command and completed properly
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;  
;  Page and Block offset must be pre-defined
;  I2C_FLASH_PAGE_OFFSET          // Zero except for 29xxx
;  I2C_FLASH_BLOCK_OFFSET
;
;  * The caller of this function should look at the return value and ACK or NAK
;    the master accordingly.
;

 EzI2Cs_Execute_CMD:
_EzI2Cs_Execute_CMD:

IF (Enable_Flash_Interface)                                          ; Enable Flash interface code

    inc  [EzI2Cs_bROM_RWcntr]                                        ; Increment command counter.
    mov  A,[EzI2Cs_bROM_RWcntr]                                      ; Calculate command index.
    sub  A,[EzI2Cs_bROM_RWoffset] 
    cmp  A,0x01                                                      ; Check for command index of 1 (The command)
    jnz  .CHK_PARAM
    mov  A,reg[EzI2Cs_DR_REG]                                        ; Get "Command" value and save it.
    mov  [EzI2Cs_bFLASH_CMD ],A                                      ; Save in command register
    jmp  .ACK_CMD

.CHK_PARAM:
    cmp  A,0x02                                                      ; Check for command index of 2 (The Param)
    jnz  .NAK_CMD                                                ; Since index is not 1 or 2, invalid, NAK data
    mov  A,reg[EzI2Cs_DR_REG]                                        ; Get "Parameter" value and save it.
    mov  [EzI2Cs_bFLASH_Param ],A                                    ; Save in Parameter register

   // Check what mode we are operating in, normal or flash mode.
   // If in normal mode, jump down to .CK_NORMAL_MODE so to skip
   // checking for flash mode commands which are invalid in normal
   // mode.
   tst  [EzI2Cs_bState],FLASH_MODE_FLAG                              ; Check if in FLASH MODE
   jz   .CK_NORMAL_MODE

   // Get command
   // Verify command by checking write pointer, command, and parameter

   // Commands


   ;=================================================================
   ;   Read Block       
   ;=================================================================

.CK_ReadBlock_CMD:
   cmp  [EzI2Cs_bFLASH_CMD],I2C_FLASH_READ_BLOCK_CMD
   jnz  .CK_WriteBlock_CMD

   // Check for block range 0 to n  (Max n-1)
   mov  A,(EzI2Cs_BlocksUsed - 1)                                    ; Load block count -1
   sub  A,[EzI2Cs_bFLASH_Param]
   jc   .NAK_CMD                                                     ; Block index out of range

   mov  X,SP                                                         ; Get copy of current Stack location
   add  [EzI2Cs_bFLASH_Param],<EzI2Cs_FirstBlock                     ; Get offset of first block


;IF (EzI2Cs_LARGE_FLASH)
IF (EzI2Cs_blockId_2bytes)
   mov  A,>EzI2Cs_FirstBlock                                         ; Get MSB of FirstBlock
   adc  A,0x00                                                       ; Add with carry to complete MSB
   push A                                                            ; Store MSB of BlockID
ENDIF


   mov  A,[EzI2Cs_bFLASH_Param]                                      ; Move LSB of blockID to A           
   push A                                                            ; Save block ID

IF (SYSTEM_LARGE_MEMORY_MODEL)
   mov  A,[EzI2Cs_pRAM_Buf_Addr_MSB]                                 ; Save RAM buffer pointer MSB
ELSE
   mov  A,0x00
ENDIF

   push A
   mov  A,[EzI2Cs_pRAM_Buf_Addr_LSB]                                 ; Save RAM buffer pointer LSB
   push A

;IF (EzI2Cs_LARGE_FLASH)
IF (EzI2Cs_ReadCount_2bytes)
   mov  A,0  // 64 bytes, entire block
   push A
ENDIF

   mov  A,EzI2Cs_FlashBlockSize  // 64 bytes, entire block
   push A

   // Setup parameters for call
   // LSB to structure is already in X
IF (SYSTEM_LARGE_MEMORY_MODEL)
   mov  A,reg[STK_PP]
ELSE
   mov  A,0x00     // Save MSB
ENDIF

   lcall FlashReadBlock
;IF (EzI2Cs_LARGE_FLASH)
;   add   SP,-6     // Fixup the stack
;ELSE
;   add   SP,-4     // Fixup the stack
;ENDIF
   add   SP,-FLASH_READ_STRUCT_SIZE 
   jmp  .ACK_CMD                                                     ; Command executed without errors.


   ;=================================================================
   ;   Write Block       
   ;=================================================================
.CK_WriteBlock_CMD:
   cmp  [EzI2Cs_bFLASH_CMD],I2C_FLASH_WRITE_BLOCK_CMD
   jnz  .CK_Reset_CMD
   // Check for block range 0 to n  (Max n-1)
   mov  A,(EzI2Cs_BlocksUsed - 1)                                    ; Load block count -1
   sub  A,[EzI2Cs_bFLASH_Param]
   jc   .NAK_CMD                                                     ; Block index out of range

   mov  X,SP                                                         ; Get copy of current Stack location
   add  [EzI2Cs_bFLASH_Param],<EzI2Cs_FirstBlock                     ; Get offset of first block

;IF (EzI2Cs_LARGE_FLASH)
IF (EzI2Cs_blockId_2bytes)
   mov  A,>EzI2Cs_FirstBlock                                         ; Get MSB of FirstBlock
   adc  A,0x00                                                       ; Add with carry to complete MSB
   push A                                                            ; Store MSB of BlockID
ENDIF

   mov  A,[EzI2Cs_bFLASH_Param]                                      ; Move LSB of blockID to A           
   push A                                                            ; Save block ID
   
IF (SYSTEM_LARGE_MEMORY_MODEL)
   mov  A,[EzI2Cs_pRAM_Buf_Addr_MSB]                                 ; Save RAM buffer pointer MSB
ELSE
   mov  A,0x00
ENDIF
   push A
   mov  A,[EzI2Cs_pRAM_Buf_Addr_LSB]                                 ; Save RAM buffer pointer LSB
   push A
   mov  A,I2C_FLASH_DEFAULT_TEMP                                     ; Use default temperature
   push A

   mov  A,0x55   // Three byte temp storage
   push A
   push A
   push A 


   // Setup parameters for call
   // LSB to structure is already in X
IF (SYSTEM_LARGE_MEMORY_MODEL)
   mov  A,reg[STK_PP]
ELSE
   mov  A,0x00     // Save MSB
ENDIF


   lcall bFlashWriteBlock
;IF (EzI2Cs_LARGE_FLASH)
;   add   SP,-8     // Fixup the stack
;ELSE
;   add   SP,-7     // Fixup the stack
;ENDIF
   add   SP,-FLASH_WRITE_STRUCT_SIZE 
  //  Translate user block to system block address
   //  Call flash write routine.
   //  Set status byte RAM_Buffer[64]
   //  Release I2C bus (ACK/NAK)
   jmp  .ACK_CMD                                                     ; Command executed without errors.

   ;=================================================================
   ;   Reset             
   ;=================================================================
.CK_Reset_CMD:
   cmp  [EzI2Cs_bFLASH_CMD],I2C_FLASH_RESET_CMD                      ; Check for Reset command.
   jnz  .CK_EnterFlashMode_CMD

   cmp  [EzI2Cs_bFLASH_Param],I2C_FLASH_RESET_KEY1
   jnz  .NAK_CMD                                                     ; Key doesn't match, bail  

   cmp  [EzI2Cs_bROM_RWoffset],I2C_FLASH_RESET_KEY0
   jnz  .NAK_CMD                                                     ; Data Pointer "Key2" doesn't match, bail  

   // Release I2C bus (ACK/NAK)
   SetEzI2Cs_SCR ( EzI2Cs_SCR_ACK )                                  ; ACK Command
   ljmp  0x0000              ; Reset


.CK_NORMAL_MODE:

   ;=================================================================
   ;   Enter Flash Mode
   ;=================================================================
.CK_EnterFlashMode_CMD:
   cmp  [EzI2Cs_bFLASH_CMD],I2C_ENTER_FLASH_MODE_CMD                 ; Check for RESET command
   jnz  .CK_ShowBlock_CMD
   cmp  [EzI2Cs_bFLASH_Param],I2C_ENTER_FLASH_MODE_KEY1
   jnz  .NAK_CMD                                                     ; Key doesn't match, bail  
   cmp  [EzI2Cs_bROM_RWoffset],I2C_ENTER_FLASH_MODE_KEY0
   jnz  .NAK_CMD                                                     ; Data Pointer "Key2" doesn't match, bail  
   // Clear all interrupts except I2C interrupt
    mov  reg[INT_MSK0],0x00
    mov  reg[INT_MSK1],0x00
	mov  reg[INT_MSK2],0x00


   // Create 65 byte buffer is free space
   // Move stack in clear area

   mov  A,< EzI2Cs_varPage                                           ; Get LSB of variable
   and  A,0xC0                                                       ; Mask off all but 2 MSb
.REGION0:                                                       ; I2C UM memory between 0x00 - 0x3F
   cmp  A,0x00
   jnz  .REGION1
   mov  [EzI2Cs_pRAM_Buf_Addr_LSB],0xA0                              ; Place RAM Buffer 0xA0
   mov  A,0x58                                                       ; Place Stack at   0x58
   jmp  .SET_STACK

.REGION1:                                                       ; I2C UM memory between 0x40 - 0x7F
   cmp  A,0x40
   jnz  .REGION2
   mov  [EzI2Cs_pRAM_Buf_Addr_LSB],0xB0                              ; Place RAM Buffer 0xB0
   mov  A,0x00                                                       ; Place Stack at   0x00
   jmp  .SET_STACK

.REGION2:                                                       ; I2C UM memory between 0x80 - 0xBF
   cmp  A,0x80
   jnz  .REGION3
   mov  [EzI2Cs_pRAM_Buf_Addr_LSB],0x00                              ; Place RAM Buffer 0x00
   mov  A,0x50                                                       ; Place Stack at   0x50
   jmp  .SET_STACK

.REGION3:                                                       ; I2C UM memory between 0xC0 - 0xFF
   mov  [EzI2Cs_pRAM_Buf_Addr_LSB],0x00                              ; Place RAM Buffer 0x00
   mov  A,0x80                                                       ; Place Stack at   0x80
  
.SET_STACK:
    swap A,SP

.SET_BUF_SIZE:   
   mov  [EzI2Cs_bRAM_Buf_Size], EzI2Cs_RAMBufferSize               ; Set buffer to 65 bytes
   mov  [EzI2Cs_bRAM_Buf_WSize],EzI2Cs_FlashBlockSize              ; Set R/W area to 64 bytes
   mov  [EzI2Cs_bROM_Buf_Size], EzI2Cs_FlashBlockSize             ; Make sure ROM window is 64 bytes
 
 
IF (SYSTEM_LARGE_MEMORY_MODEL)
   REG_RESTORE CUR_PP           ; Restore Current Page Pointer
   REG_RESTORE IDX_PP           ; Restore Index Page Pointer
ENDIF
  
   // preload the stack so that an rti will take program
   // execution to the Flash main routine.
//   RAM_CHANGE_PAGE_MODE( FLAG_PGMODE_11b )  // 00
   mov  A,>EzI2Cs_FLASH_Main                                         ; Push PCH
   push A
   mov  A,<EzI2Cs_FLASH_Main                                         ; Push PCL
   push A
   mov  A,0xC1                                                       ; Push Flags, with Interrupts enabled
   push A

   // Clear stack (force return to FLASH_MAIN)
   // Release I2C bus (ACK/NAK)
   or [EzI2Cs_bState],FLASH_MODE_FLAG                                ; Set FLASH mode bit
   SetEzI2Cs_SCR ( EzI2Cs_SCR_ACK )                                  ; Release I2C bus (ACK/NAK)
   
   reti    // Return to Flash Main, NOT where we came from!!!


   ;=================================================================
   ;   Show Block
   ;=================================================================
.CK_ShowBlock_CMD:
   cmp  [EzI2Cs_bFLASH_CMD],I2C_FLASH_SHOW_BLOCK_CMD
   jnz  .NAK_CMD
   // Check for block range 0 to n  (Max n-1)
   mov  A,(EzI2Cs_BlocksUsed - 1)                                    ; Load block count -1
   sub  A,[EzI2Cs_bFLASH_Param]
   jc   .NAK_CMD                                                     ; Block index out of range

   mov  [EzI2Cs_pROM_Buf_Addr_MSB],>EzI2Cs_FirstBlock                ; Put MSB of start block ID
                                                                     ; in MSB of Buffer address.
   mov  A,<EzI2Cs_FirstBlock                                         ; Get LSB of first block
   add  A,[EzI2Cs_bFLASH_Param]                                      ; Add the block offset (parameter)
   mov  [EzI2Cs_pROM_Buf_Addr_LSB],A                                 ; Load the LSB of the ROM pointer
   adc  [EzI2Cs_pROM_Buf_Addr_MSB],0x00                              ; Update MSB of buffer address (add with carry)

   
   asl  [EzI2Cs_pROM_Buf_Addr_LSB]                                   ; Convert from blocks to address (mult by 64)
   rlc  [EzI2Cs_pROM_Buf_Addr_MSB]                                   ; Shift left 6 times.

   asl  [EzI2Cs_pROM_Buf_Addr_LSB]
   rlc  [EzI2Cs_pROM_Buf_Addr_MSB]

   asl  [EzI2Cs_pROM_Buf_Addr_LSB]
   rlc  [EzI2Cs_pROM_Buf_Addr_MSB]

   asl  [EzI2Cs_pROM_Buf_Addr_LSB]
   rlc  [EzI2Cs_pROM_Buf_Addr_MSB]

   asl  [EzI2Cs_pROM_Buf_Addr_LSB]
   rlc  [EzI2Cs_pROM_Buf_Addr_MSB]

   asl  [EzI2Cs_pROM_Buf_Addr_LSB]
   rlc  [EzI2Cs_pROM_Buf_Addr_MSB]

   mov  [EzI2Cs_bROM_Buf_Size],EzI2Cs_FlashBlockSize                                  ; Make sure ROM window is 64 bytes
   jmp  .ACK_CMD                                                     ; Command executed without errors.



.NAK_CMD:
   mov  A,0x00   // Command good
   jmp  EzI2Cs_Execute_CMD_End

.ACK_CMD:
   mov  A,0x01   // Command good

 EzI2Cs_Execute_CMD_End:
   ret



;-----------------------------------------------------------------------------
;  FUNCTION NAME: EzI2Cs_Flash_Main
;
;  DESCRIPTION:
;     Execute flash command
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: none
;
;  RETURNS: none
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;

 EzI2Cs_FLASH_Main:

// Use only page zero
// Calculate where Flash RAM array can exist
// Create 64 byte array plus 1 status byte  ( 65 bytes )
// Set I2C RAM pointer to above array
// Move 

TO_CNTR_MSB:  equ  0
TO_CNTR_ISB:  equ  1
TO_CNTR_LSB:  equ  2

   
   mov  X,SP      // Clear the time-out counter
   add  SP,3      // Add three byte for a variable


   // Time-Out counter will

.INIT_TO_CNTR:   // Init Timeout counter to 0x00000000
                 // Reset if timer gets to  0x01000000
   mov  [X+TO_CNTR_MSB], 0xE8
   mov  [X+TO_CNTR_ISB], 0x00
   mov  [X+TO_CNTR_LSB], 0x00  
   

.Flash_Loop:  
   nop
   nop                                                       ; Do nothing loop
IF (I2C_FLASH_AUTO_TIMEOUT)
   push   X
   lcall  EzI2Cs_GetActivity
   pop    X
   cmp    A,0x00
   jz     .IncTimer
   jmp    .INIT_TO_CNTR

.IncTimer:

   mov    A,0
   inc    [X+TO_CNTR_LSB]
   adc    [X+TO_CNTR_ISB],A
   adc    [X+TO_CNTR_MSB],A

   jnc   .Flash_Loop

   M8C_Reset                                                     ; Jump back to Reset Vector

ELSE
   jnc   .Flash_Loop
ENDIF

ELSE
   mov  A,0x00          ; Invalid command
   ret                  ; If Flash not used, just return
ENDIF

ELSE  ; Flash interface not enabled
   ret
ENDIF
; end of file EzI2CsINT.asm
