//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>          // part specific constants and macros
#include "PSoCAPI.h"      // PSoC API definitions for all User Modules

#include "driverdecl.h"
#include "CMXSystem.h"
#include "CMXSystemExtern.h"
#include "TransferFunction.h"

#include "cmx.h"
#include "ProjectProperties.h"
#include "Custom.h"

// Channel includes
// CSD_00 Include
#include "CMX_CSDR_CHAN.h"
// SWPWM_Timer_01 Include
#include "CMX_SWPWM_CHAN.h"
// USBUART_SHARED_00 Include
#include "cmx_USBUART_chan.h"

void main()
{
    // Initialize Project
    M8C_EnableGInt;                            // Turn on interrupts 
    SystemTimer_Start();
    SystemTimer_SetInterval(SystemTimer_64_HZ);
    SystemTimer_EnableInt();

    // Initialize Channels
    // CSD_00 Initialization  
	CSD_Start();
    // set I2C block pins to high
    PRT1DR |= 1 << 0;
    PRT1DR |= 1 << 1;
    // SWPWM_Timer_01 Initialization
    InitSWPWM();
    // USBUART_SHARED_00 Initialization
    USBUARTStartSharedChan();
    I2C_CFG &= 0xFC;                           // Disable I2C in case it's not used.
    
    // Initialize Variables
	SystemVars.ReadOnlyVars.pse_LED_XM = 100;
	SystemVars.ReadOnlyVars.pse_LED_XP = 0;
	SystemVars.ReadOnlyVars.pse_LED_YM = 100;
	SystemVars.ReadOnlyVars.pse_LED_YP = 0;
	SystemVars.ReadOnlyVars.pse_PWM = 0;
	SystemVars.ReadOnlyVars.pse_ScaleDetect = 0;
	SystemVars.ReadOnlyVars.pse_Slide_X_Position = 0;
	SystemVars.ReadOnlyVars.pse_Slide_X_Tuning0_Difference = 0;
	SystemVars.ReadOnlyVars.pse_Slide_X_Tuning1_Difference = 0;
	SystemVars.ReadOnlyVars.pse_Slide_X_Tuning2_Difference = 0;
	SystemVars.ReadOnlyVars.pse_Slide_X_Tuning3_Difference = 0;
	SystemVars.ReadOnlyVars.pse_Slide_X_Tuning4_Difference = 0;
	SystemVars.ReadOnlyVars.pse_Slide_Y_Position = 0;
	SystemVars.ReadOnlyVars.pse_Slide_Y_Tuning0_Difference = 0;
	SystemVars.ReadOnlyVars.pse_Slide_Y_Tuning1_Difference = 0;
	SystemVars.ReadOnlyVars.pse_Slide_Y_Tuning2_Difference = 0;
	SystemVars.ReadOnlyVars.pse_Slide_Y_Tuning3_Difference = 0;
	SystemVars.ReadOnlyVars.pse_Slide_Y_Tuning4_Difference = 0;

    // Driver instantiations
CMX_CSDRPROPERTY_Instantiate(&pse_CSDProp);
CMX_I2C_Instantiate(&pse_I2C_Slave);
CMX_VarFreqDutyPWM_Instantiate(&pse_PWM);
CMX_SWINTCONTLED_Instantiate(&pse_LED_XP);
CMX_SWINTCONTLED_Instantiate(&pse_LED_XM);
CMX_SWINTCONTLED_Instantiate(&pse_LED_YP);
CMX_SWINTCONTLED_Instantiate(&pse_LED_YM);
CMX_USBUART_Instantiate(&pse_USB_COM);
CMX_CSDRSLIDER_Instantiate(&pse_Slide_X);
CMX_CSDRSLIDER_Instantiate(&pse_Slide_Y);

    // Custom initization code.
    CustomInit();
    // End Initialize Project

	while(1)
	{
        // Sync loop sample rate
#if ( SAMPLE_DIVIDER  )
        SystemTimer_SyncWait(SAMPLE_DIVIDER, SystemTimer_WAIT_RELOAD);
#endif

		// update input variables
		SystemVars.ReadOnlyVars.pse_Slide_X_Position = CMX_CSDRSLIDER_GetValue(&pse_Slide_X);
#if 1
			SystemVars.ReadOnlyVars.pse_Slide_X_Tuning0_Difference = CMX_CSDRSLIDER_GetDifference(&pse_Slide_X_CSD_Slider00);
	SystemVars.ReadOnlyVars.pse_Slide_X_Tuning1_Difference = CMX_CSDRSLIDER_GetDifference(&pse_Slide_X_CSD_Slider01);
	SystemVars.ReadOnlyVars.pse_Slide_X_Tuning2_Difference = CMX_CSDRSLIDER_GetDifference(&pse_Slide_X_CSD_Slider02);
	SystemVars.ReadOnlyVars.pse_Slide_X_Tuning3_Difference = CMX_CSDRSLIDER_GetDifference(&pse_Slide_X_CSD_Slider03);
	SystemVars.ReadOnlyVars.pse_Slide_X_Tuning4_Difference = CMX_CSDRSLIDER_GetDifference(&pse_Slide_X_CSD_Slider04);

#endif
		SystemVars.ReadOnlyVars.pse_Slide_Y_Position = CMX_CSDRSLIDER_GetValue(&pse_Slide_Y);
#if 1
			SystemVars.ReadOnlyVars.pse_Slide_Y_Tuning0_Difference = CMX_CSDRSLIDER_GetDifference(&pse_Slide_Y_CSD_Slider00);
	SystemVars.ReadOnlyVars.pse_Slide_Y_Tuning1_Difference = CMX_CSDRSLIDER_GetDifference(&pse_Slide_Y_CSD_Slider01);
	SystemVars.ReadOnlyVars.pse_Slide_Y_Tuning2_Difference = CMX_CSDRSLIDER_GetDifference(&pse_Slide_Y_CSD_Slider02);
	SystemVars.ReadOnlyVars.pse_Slide_Y_Tuning3_Difference = CMX_CSDRSLIDER_GetDifference(&pse_Slide_Y_CSD_Slider03);
	SystemVars.ReadOnlyVars.pse_Slide_Y_Tuning4_Difference = CMX_CSDRSLIDER_GetDifference(&pse_Slide_Y_CSD_Slider04);

#endif

        // Custom Post Input function
        CustomPostInputUpdate();

		// run transfer function and update output variables
		TransferFunction();

        // CustomPreOutputUpdate();
        CustomPreOutputUpdate();

		// set outputs
		CMX_SWINTCONTLED_SetValue(&pse_LED_XM, (BYTE)SystemVars.ReadOnlyVars.pse_LED_XM);
		CMX_SWINTCONTLED_SetValue(&pse_LED_XP, (BYTE)SystemVars.ReadOnlyVars.pse_LED_XP);
		CMX_SWINTCONTLED_SetValue(&pse_LED_YM, (BYTE)SystemVars.ReadOnlyVars.pse_LED_YM);
		CMX_SWINTCONTLED_SetValue(&pse_LED_YP, (BYTE)SystemVars.ReadOnlyVars.pse_LED_YP);
		CMX_VarFreqDutyPWM_SetValue(&pse_PWM, (int)SystemVars.ReadOnlyVars.pse_PWM);
		CMX_USBUART_DataTransfer(&pse_USB_COM); 
	}
}
