//*****************************************************************************
//*****************************************************************************
//  FILENAME: CSD.h
//  Version: 1.3, Updated on 2009/5/7 at 21:9:1
//  Generated by PSoC Designer 5.0.423.0
//
//  DESCRIPTION: CSD User Module C Language interface file for the 24x94 PSoC
//               family of devices
//-----------------------------------------------------------------------------
//  Copyright (c) Cypress Semiconductor 2006-2007. All Rights Reserved.
//*****************************************************************************
//*****************************************************************************

#include <m8c.h>

//--------------------------------------------------
// Constants for CSD
//--------------------------------------------------

#define CSD_TotalSensorCount                      0xa
#define CSD_ButtonCount                           0x0
#define CSD_SliderCount                           0x2

#define CSD_BUTTONS_EXIST                         0x0
#define CSD_SLIDERS_EXIST                         0x2

//--------------------------------------------------
// Constants for Scan Speed values
//--------------------------------------------------
#define CSD_FAST_SPEED                            0x01
#define CSD_NORMAL_SPEED                          0x02
#define CSD_SLOW_SPEED                            0x03
#define CSD_SCANNING_SPEED                        0x2

//--------------------------------------------------
// Constants for Thresholds values
//--------------------------------------------------
#define CSD_FINGER_THRESHOLD                      0x64
#define CSD_NOISE_THRESHOLD                       0x28
#define CSD_BASELINE_UPDATE_THRESHOLD             0xc8
#define CSD_HYSTERESIS                            0xa

//--------------------------------------------------
// Constants for Resolution values
//--------------------------------------------------
#define CSD_MIN_RESOLUTION                        8
#define CSD_MAX_RESOLUTION                        12
#define CSD_RESOLUTION                           0xc

//--------------------------------------------------
// Variables for CSD.
//--------------------------------------------------
extern BYTE CSD_bNoiseThreshold;
extern BYTE CSD_bNegativeNoiseThreshold;
extern BYTE CSD_bBaselineUpdateThreshold;
extern BYTE CSD_bHysteresis;
extern BYTE CSD_bDebounce;
extern BYTE CSD_bLowBaselineReset;

extern WORD CSD_waSnsResult[];
extern WORD CSD_waSnsBaseline[];
extern WORD CSD_waSnsDiff[];
extern BYTE CSD_baSnsOnMask[];

#if CSD_BUTTONS_EXIST
extern BYTE CSD_baBtnFThreshold[];
#endif

// Low Level scan API
#pragma fastcall16 CSD_Start
#pragma fastcall16 CSD_Stop
#pragma fastcall16 CSD_ScanSensor
#pragma fastcall16 CSD_ScanAllSensors
#pragma fastcall16 CSD_ClearSensors
#pragma fastcall16 CSD_wReadSensor
#pragma fastcall16 CSD_wGetPortPin
#pragma fastcall16 CSD_EnableSensor
#pragma fastcall16 CSD_DisableSensor
#pragma fastcall16 CSD_SetScanMode
#pragma fastcall16 CSD_SetRefValue

// High Level API
#pragma fastcall16 CSD_UpdateSensorBaseline
#pragma fastcall16 CSD_bIsSensorActive
#pragma fastcall16 CSD_bIsAnySensorActive
#pragma fastcall16 CSD_SetDefaultFingerThresholds
#pragma fastcall16 CSD_InitializeSensorBaseline
#pragma fastcall16 CSD_InitializeBaselines
#pragma fastcall16 CSD_UpdateAllBaselines
#pragma fastcall16 CSD_wGetCentroidPos

//-------------------------------------------------
// C prototypes of the CSD API.
//-------------------------------------------------

extern void CSD_Start(void);
extern void CSD_Stop(void);
extern void CSD_ScanSensor(BYTE bSensor);
extern void CSD_ScanAllSensors(void);
extern void CSD_ClearSensors(void);
extern WORD CSD_wReadSensor(BYTE bSensor);
extern WORD CSD_wGetPortPin(BYTE bSensor);
extern void CSD_EnableSensor(BYTE bMask, BYTE bPort);
extern void CSD_DisableSensor(BYTE bMask, BYTE bPort);
extern void CSD_SetScanMode(BYTE bSpeed, BYTE bResolution);
extern void CSD_SetRefValue(BYTE bRefValue);

//-------------------------------------------------
// High level functions
//-------------------------------------------------

extern void CSD_UpdateSensorBaseline(BYTE bSensorNum);
extern BYTE CSD_bIsSensorActive(BYTE bSensorNum);
extern BYTE CSD_bIsAnySensorActive(void);
extern void CSD_SetDefaultFingerThresholds(void);
extern void CSD_InitializeSensorBaseline(BYTE bSensorNum);
extern void CSD_InitializeBaselines(void);
extern void CSD_UpdateAllBaselines(void);
#if CSD_SLIDERS_EXIST
extern WORD CSD_wGetCentroidPos(BYTE bSnsGroup);
#endif


//-------------------------------------------------
// Register Addresses for CSD
//-------------------------------------------------
#pragma ioport  CSD_PRS_FUNC_REG:   0x120                  // PRS Function register
BYTE            CSD_PRS_FUNC_REG;
#pragma ioport  CSD_PRS_IN_REG: 0x121                      // PRS Input register
BYTE            CSD_PRS_IN_REG;
#pragma ioport  CSD_PRS_OUT_REG:    0x122                  // PRS Output register
BYTE            CSD_PRS_OUT_REG;
#pragma ioport  CSD_PRS_SHIFT_REG:  0x020                  // PRS Shift register
BYTE            CSD_PRS_SHIFT_REG;
#pragma ioport  CSD_PRS_POLY_REG:   0x021                  // PRS Polynomial register
BYTE            CSD_PRS_POLY_REG;
#pragma ioport  CSD_PRS_SEED_REG:   0x022                  // PRS Seed register
BYTE            CSD_PRS_SEED_REG;
#pragma ioport  CSD_PRS_CONTROL_REG:    0x023              // PRS Control register
BYTE            CSD_PRS_CONTROL_REG;

